function [Cp,P,Ct,T,Pn,Pt,pitch,alpha,rpm,C_ad,F_ad] = BEM(rho_air, Vo, V_rated, turbineType, rpm_min, rpm_max, V_cutin, V_cutout)
% %BEM Calculates [Cp,P,Ct,T,Pn,Pt] as function of (rho, V0)
% pitch and rpm rotor derived from V0 wind speed
% %Units: Vo [m/s], w [rad/s], pitch [degrees!!]

% Thrust force T at hub height, power P, pitch, rpm and aerodynamic damping C_ad. Ignore hub motion!
% Pitch actuator is assumed to be slow, such that pitch/thrust cannot be
% corrected by including pile/hub motion. Therefore pile motion is ignored.

% Input: air density, incoming wind speed at hub height, rated rotor wind speed, 'NREL5' and its min/max rpm/operating wind speed. 
       

if turbineType ~= 'NREL5'
    error('Turbine data does not comply with the specified turbineType. Only NREL 5MW turbine can be examined.')
end

%Read airfoil data NREL 5 reference turbine
x1=importdata('Airfoils/Cylinder1.dat'); %[m,n]=size(x)
alfatab1=x1(:,1)*(pi/180);
cltab1=x1(:,2);
cdtab1=x1(:,3);

x2=importdata('Airfoils/Cylinder2.dat'); %[m,n]=size(x)
alfatab2=x2(:,1)*(pi/180);
cltab2=x2(:,2);
cdtab2=x2(:,3);

x3=importdata('Airfoils/DU21_A17.dat'); %[m,n]=size(x)
alfatab3=x3(:,1)*(pi/180);
cltab3=x3(:,2);
cdtab3=x3(:,3);

x4=importdata('Airfoils/DU25_A17.dat'); %[m,n]=size(x)
alfatab4=x4(:,1)*(pi/180);
cltab4=x4(:,2);
cdtab4=x4(:,3);

x5=importdata('Airfoils/DU30_A17.dat'); %[m,n]=size(x)
alfatab5=x5(:,1)*(pi/180);
cltab5=x5(:,2);
cdtab5=x5(:,3);

x6=importdata('Airfoils/DU35_A17.dat'); %[m,n]=size(x)
alfatab6=x6(:,1)*(pi/180);
cltab6=x6(:,2);
cdtab6=x6(:,3);

x7=importdata('Airfoils/DU40_A17.dat'); %[m,n]=size(x)
alfatab7=x7(:,1)*(pi/180);
cltab7=x7(:,2);
cdtab7=x7(:,3);

x8=importdata('Airfoils/NACA64_A17.dat'); %[m,n]=size(x)
alfatab8=x8(:,1)*(pi/180);
cltab8=x8(:,2);
cdtab8=x8(:,3);

% NREL 5 blade properties
B = 3; % number of blades
R = 63; % Blade length [m]. Assume R = D/2:  D = 2R+Dhub, Rblade = 61.5, hubD = 3
r = [2.8667 5.6 8.3333 11.75 15.85 19.95 24.05 28.15 32.25 36.35 40.45 44.55 48.65 52.75 56.1667 58.9 61.6333]./R; % =(r/R)
twist = [13.308 13.308 13.308 13.308 11.480 10.162 9.011 7.795 6.544 5.361 4.188 3.125 2.319 1.526 0.863 0.370 0.106]*(pi/180); %radians
c = [3.542 3.854 4.167 4.557 4.652 4.458 4.249 4.007 3.748 3.502 3.256 3.01 2.764 2.518 2.313 2.086 1.419]; %meters
airfoil = ["Cylinder1","Cylinder1","Cylinder2","DU40_A17","DU35_A17","DU35_A17","DU30_A17","DU25_A17","DU25_A17","DU21_A17","DU21_A17","NACA64_A17","NACA64_A17","NACA64_A17","NACA64_A17","NACA64_A17","NACA64_A17"];
A = pi*R^2; % Swept rotor area [m2]

bladepitch = [0 3.83 6.60 8.70 10.45 12.06 13.54 14.92 16.23 17.47 18.7 19.94 21.18 22.35 23.47]; % in degrees, 
%for wind speeds:
bladepitch_windspeed = [11.4 12.0 13.0 14.0 15.0 16.0 17.0 18.0 19.0 20.0 21.0 22.0 23.0 24.0 25.0]; %m/s 
%pitch_fun = polyfit(bladepitch_windspeed,bladepitch,3);

% Operational conditions
rho = rho_air; %[kg/m3]

if Vo < V_cutin
    Cp = 0;
    P = 0;
    Ct = 0;
    T = 0;
    Pn = 0;
    Pt = 0;
    pitch = 0;
    rpm = 0;
    C_ad = 0;
    return
    %error('Wind speed too low for rotor to operate (V0 < Vcutin).') 
elseif Vo <= V_rated
    pitch = 0;
    rpm = (rpm_max-rpm_min)/(V_rated - V_cutin)*Vo+(rpm_max-(rpm_max-rpm_min)/(V_rated - V_cutin)*V_rated);
elseif Vo > V_rated && Vo <= V_cutout
    rpm = 12.1;
    pitch = interp1(bladepitch_windspeed,bladepitch,Vo);
elseif Vo > V_cutout
    Cp = 0;
    P = 0;
    Ct = 0;
    T = 0;
    Pn = 0;
    Pt = 0;
    pitch = 0;
    rpm = 0;
    C_ad = 0;
    return
    %error('Wind speed too high for rotor to operate (V0 > Vcutout).')
end

w = rpm*2*pi/60; % rpm to rad/s
    
%0. Initialize vectors
fg=ones(1,length(r));

% 1. Initialize a and a'
a = zeros(1,length(r));
aprime = zeros(1,length(r));

Count = 0;
%% UPDATE
while 1
    
    % 2. Compute flow angle phi (using a, a',U, w)
    phi = atan((Vo.*(1-a))./(w.*r*R.*(1+aprime)));%radians
    arg = (-B/2)*((R-r*R)./(r*R.*sin(phi)));
    F = (2/pi).*acos(exp(arg));
    
    % 3. Compute local AoA
    alpha = phi - (pitch*(pi/180))- twist; %radians
    
    % 4. Read Cl(alpha), Cd(alpha)
    for i = 1:length(alpha)
        if airfoil(1,i) == "Cylinder1"
            Cl(:,i)=interp1(alfatab1,cltab1,alpha(:,i));
            Cd(:,i)=interp1(alfatab1,cdtab1,alpha(:,i));
        elseif airfoil(1,i) == "Cylinder2"
            Cl(:,i)=interp1(alfatab2,cltab2,alpha(:,i));
            Cd(:,i)=interp1(alfatab2,cdtab2,alpha(:,i));
        elseif airfoil(1,i) == "DU21_A17"
            Cl(:,i)=interp1(alfatab3,cltab3,alpha(:,i));
            Cd(:,i)=interp1(alfatab3,cdtab3,alpha(:,i));
        elseif airfoil(1,i) == "DU25_A17"
            Cl(:,i)=interp1(alfatab4,cltab4,alpha(:,i));
            Cd(:,i)=interp1(alfatab4,cdtab4,alpha(:,i));
        elseif airfoil(1,i) == "DU30_A17"
            Cl(:,i)=interp1(alfatab5,cltab5,alpha(:,i));
            Cd(:,i)=interp1(alfatab5,cdtab5,alpha(:,i));
        elseif airfoil(1,i) == "DU35_A17"
            Cl(:,i)=interp1(alfatab6,cltab6,alpha(:,i));
            Cd(:,i)=interp1(alfatab6,cdtab6,alpha(:,i));
        elseif airfoil(1,i) == "DU40_A17"
            Cl(:,i)=interp1(alfatab7,cltab7,alpha(:,i));
            Cd(:,i)=interp1(alfatab7,cdtab7,alpha(:,i));
        elseif airfoil(1,i) == "NACA64_A17"
            Cl(:,i)=interp1(alfatab8,cltab8,alpha(:,i));
            Cd(:,i)=interp1(alfatab8,cdtab8,alpha(:,i));
        else
            error('The airfoil data is unknown.')
        end
    end
    
    % 5. Compute Cn and Ct
    Cn = Cl .* cos(phi) + Cd .* sin(phi);
    Ct = Cl.*sin(phi) -Cd.*cos(phi);
    
    %% Update a %%
    %Calculate Ct
    sig = (c.*B)./(2*pi.*r*R);
    
    CT= ((1-a).^2 .*Cn.*sig) ./(sin(phi).^2);
    
    for i=1:length(r)
        if a(i)<=1/3
            fg(i) = 1;
        else
            fg(i)=0.25*(5-3*a(i));
        end
    end
    
    anew = 0.1* CT./(4*F.*(1-fg.*a)) +0.9*a;
    %
    % CT=4*a.*F.*(1-(a.*fg));
    %
    % %Create look up table
    %
    % ax=0:0.0001:1;
    % CTx=zeros(length(ax),length(r));
    % for j=1:length(ax)
    % sigma = (c.*B)./(2*pi.*r*R);
    % CTx(j,3:length(r))= (1-ax(j)).^2 .*Cn.*sigma ./(sin(phi).^2);
    %
    % end
    % for i=1:length(a)
    % anew(i)= 0.9*a(i)+0.1*(interp1(CTx(:,i),ax,CT(i)));
    % end
    
    aprimenew = 1./((4*F.*sin(phi).*cos(phi))./(sig.*Ct)-1);
    
    % 7. Compare the change in a and a' with certain tolerance
    cond1 = anew - a;
    cond2 = aprimenew - aprime;
    
    
    if max(abs(cond1))<1e-5 && max(abs(cond2))<1e-5 %tolerance levels
        break
    else
        a = anew;
        aprime = aprimenew;
        continue
        
    end
    Count = Count+1;
end

%% Calculate loads + results %%

% 8. Compute local loads
Vrel = sqrt((Vo - a.*Vo).^2 + (aprime*w.*r*R + w*r*R).^2);

Pn = 0.5*rho.*(Vrel.^2) .*c .* Cn;
Pt = 0.5*rho.*(Vrel.^2) .*c .* Ct;

% 9. Compute total power and thrust
T = trapz(r*R,Pn)*B;

M = trapz(r*R,Pt.*r.*R)*B;

P = w*M;

Ct= T/(0.5*rho*Vo^2*A);
Cp = P/(0.5*rho*Vo^3*A);

%% Compute arodynamic damping at hub due to operating rotor (Valamanesh2014)
d_alpha = alpha(2:end) - alpha(1:end-1);
dCl_alpha = (Cl(2:end)-Cl(1:end-1))./d_alpha;
dCl_alpha = [0 dCl_alpha]; % pad to same length Cl and Cd
dCd_alpha = (Cd(2:end)-Cd(1:end-1))./d_alpha;
dCd_alpha = [0 dCd_alpha]; % pad to same length Cl and Cd

Afun = rho_air.*Vo.*(1-a).*(Cl.*cos(phi)+Cd.*sin(phi)).*c;
Bfun = 0.5.*rho_air.*rpm.*r.*(1+aprime).*((dCl_alpha+Cd).*cos(phi)+(dCd_alpha-Cl).*sin(phi)).*c;

An = trapz(r*R,Afun);
Bn = trapz(r*R,Bfun);

%######## NOW WITH CORRECT COMPUTATION AERODYNAMIC DAMPING CONSTANT TO
%ENSURE 1 - 5% MODAL DAMPING RATIO BY AERODYNAMIC ROTOR DAMPING
C_ad = Bn*(An+Bn); % damping coefficient
F_ad = trapz(r*R,B*(An+Bn)*Vo*(1-a)); % added damping force moving rotor
end


