function [p_transition,y_transition,CR,y50,c,n,p_BLGR] = BoulangerTuning(G,S,FEA,p_ult_API,p_API,y_API)
% Input: p_ult_API is the API recommended p_ult, p_API is the API
% recommended p-y curve, y_API the original shaft displacement

% y should be size 76 x time, but should only hold the positive
% displacement values!!
indexstart = find(y_API(1,:)==0);
y = y_API(:,indexstart:end);
p = p_API(:,indexstart:end);
%% Virgin loading of the BLGR element: Elastic and plastic spring tuning

%% Compute Cr, p_transition and y_transition - Limits of the elastic zone
% The gradient between each set of two neighbouring data points is:
% Soil stiffness with pile displacement:
kf = SoilStiffnessConstant(FEA,p,y);
% Initial soil stiffness
kf_0 = zeros(size(kf,1),1);
y_1 = y(1,2) - y(1,1); % [m] Elastic displacement
for i = 1:FEA.N_total
    GI = griddedInterpolant(y(i,:),kf(i,:));
    kf_0(i,:) = GI(y_1);
end

% The allowed percentage difference between two consecutive gradients:
R = 0.99;

p_transition = zeros(size(kf_0));
y_transition = zeros(size(kf_0));
for i = 1:size(y,1)
    for j = 1:size(y,2)
        if isnan(kf(i,j)/kf_0(i))==1
            p_transition(i,:) = 0;
            y_transition(i,:) = 0;
        else
            if kf(i,j)/kf_0(i)<=R
                p_transition(i,:) = p(i,j);
                y_transition(i,:) = y(i,j);
                break
            end
        end
    end
end

CR = p_transition./p_ult_API; %Elastic spring active from -CR*p_ult_API to CR*p_ult_API
for i = 1:length(kf_0)
    if isnan(CR(i))==1
        CR(i) = 0;
    end
end

%% Find y50 for all soil depths
p50 = p_ult_API/2;

y50 = zeros(size(kf_0));
for i = 1:length(kf_0)
    if i<=(FEA.N_total - FEA.E_embed) % not embedded or at mudline
        y50(i,:) = 0;
    else
        [x index] = unique(p(i,:));
        y50(i,:) = interp1(p(i,index),y(i,index),p50(i),'pchip');
        % Not need to include 'extrap' when using 'pchip' (inherently included
        % in pchip). Extrapolation is needed as y50 value may be outside the
        % range of y.
    end
end

% Replace clay y50 values:
y50_clay = 2.5*G.Clay.e50*S.D_mp_1;
for z = 1:size(y,1)
    if FEA.H_n(z)<= G.Sand.d && FEA.H_n(z)>G.Clay.d
        y50(z,:) = y50_clay;
    end
end

%% Tune c (tangent) and n (sharpness) - P-y plastic spring
% Use optimization that minimizes the sum of squared errors
% At and above the mudline, the soil resistance is zero, thus so is the 
% tangent and curvature:
c = zeros(size(kf_0));
n = zeros(size(kf_0));
for z = 1:size(y,1)
    if z<=(FEA.N_total - FEA.E_embed) % not embedded or at mudline
    c(z,:) = 0;
    n(z,:) = 0;
    else
        t_data = y(z,find(y(z,:)==y_transition(z)):end);
        y_data = p(z,find(p(z,:)==p_transition(z)):end);
        fun = @(x,t_data)(p_ult_API(z) - (p_ult_API(z) - p_transition(z)).*(x(1)*y50(z)./(x(1)*y50(z)+abs(t_data-y_transition(z)))).^x(2));
        x0 = [0.5;2];
        ub = [20,10];
        lb = [0,0];
        options = optimset('Display','none');
        x = lsqcurvefit(fun,x0,t_data,y_data,lb,ub,options);
        
%         fun = @(x)sseval(x,t_data,y_data,p_ult_API(z),p_transition(z),y50(z),...
%             y_transition(z));
%         x0 = [0.5;2];
%         options = optimset('MaxFunEvals',30000,'Display','off');
%         x = fminsearch(fun,x0,options);
        c(z,:) = x(1);
        n(z,:) = x(2);
    end
end

%% Calculate the elastic and plastic spring behaviour,check with API curves
% Approximate the entire p-y behaviour by the elastic spring with CR ratio:
p_ini = zeros(size(y));
for z = 1:size(p_ini,1)
    if z<=(FEA.N_total - FEA.E_embed) % not embedded or at mudline
        p_ini(z,:) = zeros(1,size(p_ini,2));
    else
        p_ini(z,:) = CR(z)./y_transition(z).*p_ult_API(z).*y(z,:);
    end
end

p_BLGR = zeros(size(p_ini));
for i = 1:size(y,1)
    if i>(FEA.N_total - FEA.E_embed) % not embedded or at mudline
        for j = 1:size(y,2)
            % If the resulting p-value is within the elastic range:
            if round((abs(p_ini(i,j))/(CR(i)*p_ult_API(i))),5)<=1
                p_BLGR(i,j) = p_ini(i,j);
                % If it is in the plastic range, p_ini needs to be recalculated:
            elseif round((abs(p_ini(i,j))/(CR(i)*p_ult_API(i))),5)>1
                p_BLGR(i,j) = p_plastic(c(i),n(i),p_ult_API(i),p_transition(i),y50(i),y(i,j),...
                    y_transition(i));
            end
        end
    end
end

% For both directions of loading
p_BLGR = [-fliplr(p_BLGR(:,1:end-1)) p_BLGR];

% %% Check agreement API and Boulanger curves for all soil types
% f1 = figure;
% plot(y_API(find(FEA.H_n<=G.Sand.d,1)-1,:),p_BLGR(find(FEA.H_n<=G.Sand.d,1)-1,:)/1E6,'-.k',y_API(find(FEA.H_n<=G.Sand.d,1)-1,:),p_API(find(FEA.H_n<=G.Sand.d,1)-1,:)/1E6,...
%     '-k')
% set(gca,'TickLabelInterpreter','Latex','Fontsize',11)
% xlabel('Pile deflection $y$ [m]', 'Interpreter','Latex','Fontsize',11);
% ylabel('Soil resistance per unit length $p$ [MN/m]', 'Interpreter',...
%     'Latex','Fontsize',11);
% ylim([-0.8 0.8])
% hline = refline([0 0]);
% hline.Color = 'k';
% vline = line([0 0],[ylim]);
% vline.Color = 'k';
% legend({'Dynamic p-y','Static p-y'},'Location','southeast','Interpreter','Latex','Fontsize',11)
% saveas(f1,'/Users/valentinevandeputte/Documents/DATA/EWEM 3rd MSc/Thesis/Thesis/Simulations/MATLAB codes/FEA/Figures/Final Soil PY Plots (BLGR) - Report/Py_VirginLoad_Sand','epsc')
% saveas(f1,'/Users/valentinevandeputte/Documents/DATA/EWEM 3rd MSc/Thesis/Thesis/Simulations/MATLAB codes/FEA/Figures/Final Soil PY Plots (BLGR) - Report/Py_VirginLoad_Sand','fig')
% 
% f2 = figure;
% plot(y_API(find(FEA.H_n<=G.Clay.d,1)-1,:),p_BLGR(find(FEA.H_n<=G.Clay.d,1)-1,:)/1E6,'-.k',y_API(find(FEA.H_n<=G.Clay.d,1)-1,:),p_API(find(FEA.H_n<=G.Clay.d,1)-1,:)/1E6,...
%     '-k')
% set(gca,'TickLabelInterpreter','Latex','Fontsize',11)
% xlabel('Pile deflection $y$ [m]', 'Interpreter','Latex','Fontsize',11);
% ylabel('Soil resistance per unit length $p$ [MN/m]', 'Interpreter',...
%     'Latex','Fontsize',11);
% ylim([-0.4 0.4])
% hline = refline([0 0]);
% hline.Color = 'k';
% vline = line([0 0],[ylim]);
% vline.Color = 'k';
% legend({'Dynamic p-y','Static p-y'},'Location','southeast','Interpreter','Latex','Fontsize',11)
% saveas(f2,'/Users/valentinevandeputte/Documents/DATA/EWEM 3rd MSc/Thesis/Thesis/Simulations/MATLAB codes/FEA/Figures/Final Soil PY Plots (BLGR) - Report/Py_VirginLoad_Clay','epsc')
% saveas(f2,'/Users/valentinevandeputte/Documents/DATA/EWEM 3rd MSc/Thesis/Thesis/Simulations/MATLAB codes/FEA/Figures/Final Soil PY Plots (BLGR) - Report/Py_VirginLoad_Clay','fig')
% 
% f3 = figure;
% plot(y_API(find(FEA.H_n<=G.Mudstone.d,1)-1,:),p_BLGR(find(FEA.H_n<=G.Mudstone.d,1)-1,:)/1E6,'-.k',y_API(find(FEA.H_n<=G.Mudstone.d,1)-1,:),p_API(find(FEA.H_n<=G.Mudstone.d,1)-1,:)/1E6,...
%     '-k')
% set(gca,'TickLabelInterpreter','Latex','Fontsize',11)
% xlabel('Pile deflection $y$ [m]', 'Interpreter','Latex','Fontsize',11);
% ylabel('Soil resistance per unit length $p$ [MN/m]', 'Interpreter',...
%     'Latex','Fontsize',11);
% ylim([-10 10])
% hline = refline([0 0]);
% hline.Color = 'k';
% vline = line([0 0],[ylim]);
% vline.Color = 'k';
% legend({'Dynamic p-y','Static p-y'},'Location','southeast','Interpreter','Latex','Fontsize',11)
% saveas(f3,'/Users/valentinevandeputte/Documents/DATA/EWEM 3rd MSc/Thesis/Thesis/Simulations/MATLAB codes/FEA/Figures/Final Soil PY Plots (BLGR) - Report/Py_VirginLoad_Mudstone','epsc')
% saveas(f3,'/Users/valentinevandeputte/Documents/DATA/EWEM 3rd MSc/Thesis/Thesis/Simulations/MATLAB codes/FEA/Figures/Final Soil PY Plots (BLGR) - Report/Py_VirginLoad_Mudstone','fig')
% 
% f4 = figure;
% plot(y_API(find(FEA.H_n<=G.Gravel.d,1)-1,:),p_BLGR(find(FEA.H_n<=G.Gravel.d,1)-1,:)/1E6,'-.k',y_API(find(FEA.H_n<=G.Gravel.d,1)-1,:),p_API(find(FEA.H_n<=G.Gravel.d,1)-1,:)/1E6,...
%     '-k')
% set(gca,'TickLabelInterpreter','Latex','Fontsize',11)
% xlabel('Pile deflection $y$ [m]', 'Interpreter','Latex','Fontsize',11);
% ylabel('Soil resistance per unit length $p$ [MN/m]', 'Interpreter',...
%     'Latex','Fontsize',11);
% ylim([-30 30])
% hline = refline([0 0]);
% hline.Color = 'k';
% vline = line([0 0],[ylim]);
% vline.Color = 'k';
% legend({'Dynamic p-y','Static p-y'},'Location','southeast','Interpreter','Latex','Fontsize',11)
% saveas(f4,'/Users/valentinevandeputte/Documents/DATA/EWEM 3rd MSc/Thesis/Thesis/Simulations/MATLAB codes/FEA/Figures/Final Soil PY Plots (BLGR) - Report/Py_VirginLoad_Gravel','epsc')
% saveas(f4,'/Users/valentinevandeputte/Documents/DATA/EWEM 3rd MSc/Thesis/Thesis/Simulations/MATLAB codes/FEA/Figures/Final Soil PY Plots (BLGR) - Report/Py_VirginLoad_Gravel','fig')
% 
% f5 = figure;
% plot(y_API(find(FEA.H_n<=G.Sandstone.d,1)-1,:),p_BLGR(find(FEA.H_n<=G.Sandstone.d,1)-1,:)/1E6,'-.k',y_API(find(FEA.H_n<=G.Sandstone.d,1)-1,:),p_API(find(FEA.H_n<=G.Sandstone.d,1)-1,:)/1E6,...
%     '-k')
% set(gca,'TickLabelInterpreter','Latex','Fontsize',11)
% xlabel('Pile deflection $y$ [m]', 'Interpreter','Latex','Fontsize',11);
% ylabel('Soil resistance per unit length $p$ [MN/m]', 'Interpreter',...
%     'Latex','Fontsize',11);
% ylim([-25 25])
% hline = refline([0 0]);
% hline.Color = 'k';
% vline = line([0 0],[ylim]);
% vline.Color = 'k';
% legend({'Dynamic p-y','Static p-y'},'Location','southeast','Interpreter','Latex','Fontsize',11)
% saveas(f5,'/Users/valentinevandeputte/Documents/DATA/EWEM 3rd MSc/Thesis/Thesis/Simulations/MATLAB codes/FEA/Figures/Final Soil PY Plots (BLGR) - Report/Py_VirginLoad_Sandstone','epsc')
% saveas(f5,'/Users/valentinevandeputte/Documents/DATA/EWEM 3rd MSc/Thesis/Thesis/Simulations/MATLAB codes/FEA/Figures/Final Soil PY Plots (BLGR) - Report/Py_VirginLoad_Sandstone','fig')

end
