function dqdt = EOM(DOF,PHI,NumberOfModeShapes,FEA,DSS,S,EQ,G,WT,Operation,SSI,t_shutdown,t,q)
%% Modal reduction

%% Time instance
t
%% Define system
% according to modal definition, 
z = q(1:2:end,:);
zdot = q(2:2:end,:);

if size(q,1)~=2*NumberOfModeShapes
    error('Input q into ODE solver wrong format: time x nodes. Should be nodes x time.')
end

%% Instant wave elevation (reference in computation)
% Interpolate wave height
eta_grid = griddedInterpolant(EQ.t,DSS.eta); %[m]
eta = eta_grid(t);

%% Ground motion at each node
y_eq = zeros(FEA.N_total,1);
for i = 1:length(y_eq)
    if i>=FEA.N_total-FEA.E_embed % ground motion only exists at and below mudline
        y_eq_grid = griddedInterpolant(EQ.t,EQ.y_eq(i,:));
        y_eq(i,:) = y_eq_grid(t);
    end
end

dydt_eq = zeros(FEA.N_total,1);
for i = 1:length(dydt_eq)
    if i>=FEA.N_total-FEA.E_embed % ground motion only exists at and below mudline
        dydt_eq_grid = griddedInterpolant(EQ.t,EQ.dydt_eq(i,:));
        dydt_eq(i,:) = dydt_eq_grid(t);
    end
end

ddydt_eq = zeros(FEA.N_total,1);
for i = 1:length(ddydt_eq)
    if i>=FEA.N_total-FEA.E_embed % ground motion only exists at and below mudline
        ddydt_eq_grid = griddedInterpolant(EQ.t,EQ.ddydt_eq(i,:));
        ddydt_eq(i,:) = ddydt_eq_grid(t);
    end
end

%% Constant current
% Format steady current speed, constant over depth:
u_current = [zeros(FEA.E_unsubm,1);DSS.u_current.*ones(FEA.N_subm,1);zeros(FEA.E_embed,1)]; % format nodes x time 

%% Added mass due to water body submerged nodes (hydrodynamic, due to moving pile)
% The local mass matrices are:
M_local = FEA.Mt_local+FEA.Mr_local;
% Add hydrodynamic added mass/inertia:
for i = 1:1:FEA.E_total
    if i>FEA.E_unsubm && i<=FEA.E_total-FEA.E_embed % Enter submerged elements
        if (FEA.E_unsubm+FEA.N_subm-i)*FEA.l_subm<=DSS.d_hydro+eta
            M_local(i*DOF*2-3,1) = M_local(i*DOF*2-3,1) + DSS.rho_hydro*pi/4*FEA.D_outer_el(i)^2*DSS.CA;
            M_local(i*DOF*2-1,3) = M_local(i*DOF*2-1,3) + DSS.rho_hydro*pi/4*FEA.D_outer_el(i)^2*DSS.CA;
        end
    end
end
% The global mass matrix becomes
[M_global] = GlobalMatrix(FEA,M_local,DOF);
M_global(1,1) = M_global(1,1)+WT.M_RNA;

%% Nonlinear hydrodynamic drag force and water flow inertia force (no added mass moving pile)
% Initialize
f_hydro = zeros(FEA.N_total,1); % Wave load, excluding added mass moving body (in M-matrix)
u_wave = zeros(FEA.N_total,1);   % Wave velocity
dudt_wave = zeros(FEA.N_total,1);% Wave acceleration
for i = 1:1:FEA.N_total
    if i>FEA.E_unsubm && i<=FEA.N_total-FEA.E_embed % Enter submerged elements
        if (FEA.E_unsubm+FEA.N_subm-i)*FEA.l_subm<=DSS.d_hydro+eta
            % wave velocity and acceleration, and air velocity at submerged
            % nodes:
            [u_wave(i,:),dudt_wave(i,:)] = LinearAiryWave(DSS.omega_hydro,DSS.Hs,t,DSS.k_hydro,0,(FEA.E_unsubm+FEA.N_subm-i)*FEA.l_subm,DSS.d_hydro,eta);
            % Determine the hydrodynamic load at every node
            f_hydro(i,:) = HydroForce(DSS.rho_hydro, FEA.D_outer_n(i), dudt_wave(i,:), u_wave(i,:), u_current(i), DSS.CM, DSS.CD_hydro,PHI(2*i-1,:)*zdot);%ydot(i,:)
        end
        if (FEA.E_unsubm+FEA.N_subm-i)*FEA.l_subm>DSS.d_hydro+eta
            u_wave(i,:) = DSS.V_hub*log10(((FEA.E_unsubm+FEA.N_subm-i)*FEA.l_subm-DSS.d_hydro-(min(DSS.eta)))/DSS.z_0)/log10((S.h_tower_top-DSS.d_hydro-min(DSS.eta))/DSS.z_0);
            % Determine the hydrodynamic force at every node
            f_hydro(i,:) = Aeroload(DSS.rho_air,DSS.CD_air,u_wave(i,:),PHI(2*i-1,:)*zdot,FEA.D_outer_n(i)); %ydot(i,:)
        end
    end
end

%% Aerodynamic distributed load on the tower/TP
% 1) Create steady-state wind profile
% Velocity profile V_air from hub height to SWL.
% The reference height is the hub height wrt the lowest water level (LWL):
H_ref = 10; % MSL+10, wrt mudline

% 2) Aerodynamic distributed load over entire structure
% Initialize:
% Include pile motion:
f_aero = zeros(FEA.N_total,1);  
V_air = zeros(FEA.N_total,1);
for i = 1:1:FEA.N_total
    if i<=FEA.N_unsubm % Enter unsubmerged elements
    H_air = FEA.H_n(1)-(i-1)*FEA.l_unsubm - DSS.d_hydro; % height node wrt mudline
    V_air(i,:) = Velocity_Profile(DSS.V_0,H_ref,H_air,DSS.z_0);
    f_aero(i,:) = Aeroload(DSS.rho_air,DSS.CD_air,V_air(i,:),PHI(2*i-1,:)*zdot,FEA.D_outer_n(i));%ydot(i,:)
    end
end
% Exclude aero force on last node, as this is the boundary node between
% aero and hydro load, and hydro load vector contains the wind or hydro 
% load (whichever one is governing):
f_aero(FEA.N_unsubm,:) = 0;

%% Compute soil stiffness based on current pile displacement and soil motion
% Relative motion pile and soil
y_rel = PHI(1:2:end,:)*z;
% Soil stiffness from lookup table p-y curves
kf = zeros(size(y_rel));
for i = 1:FEA.N_total
    kf_grid = griddedInterpolant(G.y(i,:),G.kf_pycurve(i,:));
    kf(i,:) = kf_grid(y_rel(i,:)); 
end
% Recompute local stiffness matrices
[Kf_local] = LocalSoilStiffnessMatrix(FEA,kf,DOF);
% Assemble in global stiffness matrix
[Kf_global] = GlobalMatrix(FEA,Kf_local,DOF);
% Total stiffness matrix system
K_global = FEA.Kb_global+FEA.Ks_global+Kf_global-FEA.Kg_global-FEA.Kg0_global;

%% VISCOUS DAMPER BECOMES ACTIVE AS SOON AS THERE IS GROUND MOTION
C_global = FEA.C_global;

if strcmp(SSI,'SSI2')==1
    cv = RadiationDamper(FEA,G);
    for i = 1:FEA.N_total
        if ddydt_eq(i,:) ~= 0 % there exists ground motion
            C_global(2*i-1,2*i-1) = cv(i);
        end
    end
end
% If there is no ground motion or when using SSI1, only the rotor damping is present.

%% MODIFY FOR OPERATIONAL CONDITION
if strcmp(Operation,'Shutdown')==1
    if t>=t_shutdown % From moment first tremor is felt, the turbine performs an emergency brake. Few seconds later, the turbine produces no more power/damping/thrust -- > ~20sec
        C_global(1,1) = 0;
        T = 0;
    else
        C_global(1,1) = C_global(1,1);
        T = WT.F_ad;
    end
elseif strcmp(Operation,'Normal')==1
    C_global(1,1) = C_global(1,1);
    T = WT.F_ad;
else
    error('Error: need to select Operation as either Normal or Shutdown')
end

%% Additional external spring force due to ground motion
% % Distributed external force:
% % The updated soil stiffness:
% f_eq = kf.*y_eq; % respect format nodes x time
q_eq = zeros(size(FEA.K_global,1),1);
q_eq(1:2:end,:) = y_eq;

dqdt_eq = zeros(size(FEA.K_global,1),1);
dqdt_eq(1:2:end,:) = dydt_eq;

ddqdt_eq = zeros(size(FEA.K_global,1),1);
ddqdt_eq(1:2:end,:) = ddydt_eq;

%% TOTAL FORCE AND MOMENT AT TIME T
% Make sure to respect the reference frame sign convention
[Ft,Mt] = Force_Moment(FEA,t,T,0,f_aero+f_hydro);
% Rearrange to respect matrix order
FMt = zeros(DOF*FEA.N_total,1);
for k = 1:FEA.N_total
FMt(2*k-1,:) = Ft(k,:); % respect the matrix order
FMt(2*k,:) = Mt(k,:); % respect the matrix order
end

%% Convert 2nd order ODE to 1st order.
%
dqdt = zeros(size(q)); % format nodes x time
dqdt(1:2:end,:) = zdot;
dqdt(2:2:end,:) = (PHI'*M_global*PHI)\(PHI'*FMt+PHI'*K_global*q_eq+PHI'*C_global*dqdt_eq-PHI'*C_global*PHI*zdot-PHI'*K_global*PHI*z); % 
end