function [P_axial,Kg0_local,Kg_local] = LocalGeometricStiffnessMatrix_V2(g,WT,FEA,DOF)
% This function computes the axial load at the top of each element P_axial,
% and the local geometric stiffness matrices due to the overlying element
% (Kg0_local) and the element itself (Kg_local).
% Format 4 x 4, with q = [y1,theta1,y2,theta2].

% The axial load
P_axial = zeros(FEA.E_total,1); % weight due to overlying elements in each element

% The local geometric stiffness due to compressive weight overlying elements
Kg0_local = zeros(FEA.E_total*2*DOF,2*DOF); 
for i = 1:FEA.E_total
    % Axial load on each element caused by gravitiational pull overlying elements:
    if i ==1
        P_axial(i) = WT.M_RNA*g; % [N]
    else
        P_axial(i) = P_axial(i-1)+FEA.rho_el(i-1)*FEA.A_el(i-1)*g*FEA.L_el(i-1); % example: in element 2, the compressive load = weight RNA + weight element 1
    end
    % Gravity in pile decreases to pile base from mudline (axial bearing soil):
    if FEA.H_n(i) <= 0
        P_axial(i) = P_axial(find(FEA.H_n==0)-1) - P_axial(find(FEA.H_n==0)-1)*(i-1-find(FEA.H_n==0))/(FEA.E_total-find(FEA.H_n==0));
    end
    % The local element matrix:
    Kg0_local(4*i-3,:) = P_axial(i)/(FEA.L_el(i)*(1+FEA.phifactor(i))^2)*[6/5+2*FEA.phifactor(i)+FEA.phifactor(i)^2 -1/10*FEA.L_el(i) -(6/5+2*FEA.phifactor(i)+FEA.phifactor(i)^2) -1/10*FEA.L_el(i)];
    Kg0_local(4*i-2,:) = P_axial(i)/(FEA.L_el(i)*(1+FEA.phifactor(i))^2)*[-1/10*FEA.L_el(i) (2/15+FEA.phifactor(i)/6+(FEA.phifactor(i)^2)/12)*FEA.L_el(i)^2 1/10*FEA.L_el(i) -(1/30+FEA.phifactor(i)/6+(FEA.phifactor(i)^2)/12)*FEA.L_el(i)^2];
    Kg0_local(4*i-1,:) = P_axial(i)/(FEA.L_el(i)*(1+FEA.phifactor(i))^2)*[-(6/5+2*FEA.phifactor(i)+FEA.phifactor(i)^2) 1/10*FEA.L_el(i) 6/5+2*FEA.phifactor(i)+FEA.phifactor(i)^2 1/10*FEA.L_el(i)];
    Kg0_local(4*i,:) = P_axial(i)/(FEA.L_el(i)*(1+FEA.phifactor(i))^2)*[-1/10*FEA.L_el(i) -(1/30+FEA.phifactor(i)/6+(FEA.phifactor(i)^2)/12)*FEA.L_el(i)^2 1/10*FEA.L_el(i) (2/15+FEA.phifactor(i)/6+(FEA.phifactor(i)^2)/12)*FEA.L_el(i)^2];
end

% Local geometric stiffness due to the gravitational weight of the current element:                                
Kg_local = zeros(FEA.E_total*2*DOF,2*DOF); 
for i = 1:FEA.E_total
    if FEA.H_n(i) > 0
    % The local element matrix:
    Kg_local(4*i-3,:) = FEA.rho_el(i)*FEA.A_el(i)*g/((1+FEA.phifactor(i))^2)*[3/5+FEA.phifactor(i)+(FEA.phifactor(i)^2)/2 -(1/10+2*FEA.phifactor(i)/15+(FEA.phifactor(i)^2)/12)*FEA.L_el(i) -(3/5+FEA.phifactor(i)+(FEA.phifactor(i)^2)/2) (2*FEA.phifactor(i)/15+(FEA.phifactor(i)^2)/12)*FEA.L_el(i)];
    Kg_local(4*i-2,:) = FEA.rho_el(i)*FEA.A_el(i)*g/((1+FEA.phifactor(i))^2)*[-(1/10+2*FEA.phifactor(i)/15+(FEA.phifactor(i)^2)/12)*FEA.L_el(i) (1/30+FEA.phifactor(i)/20+(FEA.phifactor(i)^2)/24)*FEA.L_el(i)^2 (1/10+2*FEA.phifactor(i)/15+(FEA.phifactor(i)^2)/12)*FEA.L_el(i) -(1/60+FEA.phifactor(i)/12+(FEA.phifactor(i)^2)/24)*FEA.L_el(i)^2];
    Kg_local(4*i-1,:) = FEA.rho_el(i)*FEA.A_el(i)*g/((1+FEA.phifactor(i))^2)*[-(3/5+FEA.phifactor(i)+(FEA.phifactor(i)^2)/2) (1/10+2*FEA.phifactor(i)/15+(FEA.phifactor(i)^2)/12)*FEA.L_el(i) 3/5+FEA.phifactor(i)+(FEA.phifactor(i)^2)/2 -(2*FEA.phifactor(i)/15+(FEA.phifactor(i)^2)/12)*FEA.L_el(i)];
    Kg_local(4*i,:) = FEA.rho_el(i)*FEA.A_el(i)*g/((1+FEA.phifactor(i))^2)*[(2*FEA.phifactor(i)/15+(FEA.phifactor(i)^2)/12)*FEA.L_el(i) -(1/60+FEA.phifactor(i)/12+(FEA.phifactor(i)^2)/24)*FEA.L_el(i)^2 -(2*FEA.phifactor(i)/15+(FEA.phifactor(i)^2)/12)*FEA.L_el(i) (1/10+7*FEA.phifactor(i)/60+(FEA.phifactor(i)^2)/24)*FEA.L_el(i)^2];
    end
end
                                
% % Local geometric stiffness due to the gravitational weight of the current element:                                
% Kg_local = zeros(FEA.E_total*2*DOF,2*DOF); 
% for i = 1:FEA.E_total
%     % The local element matrix:
%     Kg_local(4*i-3,:) = FEA.rho_el(i)*FEA.A_el(i)*g/((1+FEA.phifactor(i))^2)*[3/5+FEA.phifactor(i)+(FEA.phifactor(i)^2)/2 (1/10+2*FEA.phifactor(i)/15+(FEA.phifactor(i)^2)/12)*FEA.L_el(i) -(3/5+FEA.phifactor(i)+(FEA.phifactor(i)^2)/2) -(2*FEA.phifactor(i)/15+(FEA.phifactor(i)^2)/12)*FEA.L_el(i)];
%     Kg_local(4*i-2,:) = FEA.rho_el(i)*FEA.A_el(i)*g/((1+FEA.phifactor(i))^2)*[(1/10+2*FEA.phifactor(i)/15+(FEA.phifactor(i)^2)/12)*FEA.L_el(i) (1/30+FEA.phifactor(i)/20+(FEA.phifactor(i)^2)/24)*FEA.L_el(i)^2 -(1/10+2*FEA.phifactor(i)/15+(FEA.phifactor(i)^2)/12)*FEA.L_el(i) -(1/60+FEA.phifactor(i)/12+(FEA.phifactor(i)^2)/24)*FEA.L_el(i)^2];
%     Kg_local(4*i-1,:) = FEA.rho_el(i)*FEA.A_el(i)*g/((1+FEA.phifactor(i))^2)*[-(3/5+FEA.phifactor(i)+(FEA.phifactor(i)^2)/2) -(1/10+2*FEA.phifactor(i)/15+(FEA.phifactor(i)^2)/12)*FEA.L_el(i) 3/5+FEA.phifactor(i)+(FEA.phifactor(i)^2)/2 (2*FEA.phifactor(i)/15+(FEA.phifactor(i)^2)/12)*FEA.L_el(i)];
%     Kg_local(4*i,:) = FEA.rho_el(i)*FEA.A_el(i)*g/((1+FEA.phifactor(i))^2)*[-(2*FEA.phifactor(i)/15+(FEA.phifactor(i)^2)/12)*FEA.L_el(i) -(1/60+FEA.phifactor(i)/12+(FEA.phifactor(i)^2)/24)*FEA.L_el(i)^2 (2*FEA.phifactor(i)/15+(FEA.phifactor(i)^2)/12)*FEA.L_el(i) (1/10+7*FEA.phifactor(i)/60+(FEA.phifactor(i)^2)/24)*FEA.L_el(i)^2];
% end
end