function [d,d_equivalent,H_n_equivalent,pu,p] = NonlinearElasticSoil_V5(G,S,FEA,y)

% ########################################################################
%  CORRECTION FOR INFINITE INITIAL SOIL STIFFNESS CLAY, MUD- AND SANDSTONE
% ########################################################################

%% Soil model I: nonlinear elastic spring model (API recommended approach)
% As this soil model concerns a soil profile in an offshore environment, it
% is assumed that all soil layers are below the water table. P-y
% formulations that agree with this assumption will be used only.

% INPUT: soil parameters G, structural parameters S, finite element
% discretization parameters FEA, pile displacement vector over time y

%% Ultimate lateral soil resistance including soil layering effect 
% According to the method proposed by Georgiadis (1983)

% The depth steps along the embedded pile:
d = transpose(0:-FEA.l_embed/100:S.h_mp_bottom); 
% REMARK: To increase accuracy of the equivalent depth calculation, it is 
% recommended to make the depth step as small as possible. It may be
% advised to use d_step = -FEA.l_embed/SCALE with SCALE an integer, such
% that the step size in the depth allows the Georgiadis integration to
% recognize the soil boundaries between layers. So d_step <= smallest 
% decimal number in FEA.l_embed. SCALE 1000 best, 100 acceptable, 10 not.

% Diameter of the pile should be constant over the embedded length
% (installation restriction):
if FEA.D_outer_n(end) == FEA.D_outer_n(FEA.E_unsubm+FEA.N_subm)
    D = FEA.D_outer_n(end);
else
    error('Pile diameter is not constant over embedment length, which is unfeasible for easy installation. Soil resistance computation incorrect. Please reiterate design.')
end

% Functions that express the ultimate soil resistance variation with depth
% LAYER 1: SAND (O'Neill and Murchison 1983)
pu_fun1 = @(d) min((G.Sand.C1.*abs(d)+G.Sand.C2*D)*G.Sand.gamma.*abs(d),G.Sand.C3*...
    D*G.Sand.gamma.*abs(d));
% LAYER 2: SOFT CLAY (Matlock 1970)
pu_fun2 = @(d) min(3*G.Clay.sigma_u*D+G.Clay.gamma.*abs(d)*D+G.Clay.J.*...
    abs(d)*G.Clay.sigma_u,9*G.Clay.sigma_u*D);
% LAYER 3: MUDSTONE (Reese et al. 1997)
iif = @(varargin) varargin{2 * find([varargin{1:2:end}], 1, 'first')}();
pu_fun3 = @(d) iif(abs(d)<=3*D, @() G.Mudstone.alpha_r*G.Mudstone.q_ur*D*...
    (1+1.4*abs(d)/D), abs(d)>3*D, @() G.Mudstone.alpha_r*G.Mudstone.q_ur*...
    D*5.2);
% LAYER 4: GRAVEL SAND (O'Neill and Murchison)
pu_fun4 = @(d) min((G.Gravel.C1.*abs(d)+G.Gravel.C2*D)*G.Gravel.gamma.*abs(d),G.Gravel.C3*...
    D*G.Gravel.gamma.*abs(d));
% LAYER 5: SANDSTONE (Reese et al. 1997)
pu_fun5 = @(d) iif(abs(d)<=3*D, @() G.Sandstone.alpha_r*G.Sandstone.q_ur*D*...
    (1+1.4*abs(d)/D), abs(d)>3*D, @() G.Sandstone.alpha_r*G.Sandstone.q_ur*...
    D*5.2);
                  
% Calculate the integral of p_ultimate for the top layer
pu_sum_layer1 = integral(pu_fun1,0,abs(G.Sand.d));

% To find the equivalent depth, the integral of p_ultimate of the top layer
% should be equal to the integral of p_ultimate of the underlying layer,
% again from ground surface to an unknown depth d_equivalent.
pu_sum_layer2 =0; % Initialize
for i = 1:length(d)
    if pu_sum_layer2<=pu_sum_layer1
        pu_sum_layer2 = integral(pu_fun2,0,abs(d(i)));
        eq_depth_layer2 = d(i);
    else
        break
    end
end

% Calculate the delta between the equivalent depth and the actual depth of
% the start of the underlying soil layer
delta_eq_layer2 = eq_depth_layer2-G.Sand.d;
% The equivalent depth with respect to ground surface for which the layered
% soil and assumed homogeneous soil have the same ultimate soil resistance
% is then:
end_updated_layer2 = delta_eq_layer2+G.Clay.d;

% Repeat the procedure: calculate the integral of the ultimate soil
% resistance of the overlying layer from ground level to the equivalent
% depth,
pu_sum_layer2 = integral(pu_fun2,0,abs(end_updated_layer2));
% and iterate to find the same integral for the underlying soil layer:
pu_sum_layer3 =0; % Initialize
for i = 1:length(d)
    if pu_sum_layer3<=pu_sum_layer2
        pu_sum_layer3 = integral(pu_fun3,0,abs(d(i)));
        eq_depth_layer3 = d(i);
    else
        break
    end
end
delta_eq_layer3 = eq_depth_layer3-G.Clay.d;
end_updated_layer3 = delta_eq_layer3+G.Mudstone.d;

% Repeat the procedure: calculate the integral of the ultimate soil
% resistance of the overlying layer from ground level to the equivalent
% depth,
pu_sum_layer3 = integral(pu_fun3,0,abs(end_updated_layer3));
% and iterate to find the same integral for the underlying soil layer:
pu_sum_layer4 =0; % Initialize
for i = 1:length(d)
    if pu_sum_layer4<=pu_sum_layer3
        pu_sum_layer4 = integral(pu_fun4,0,abs(d(i)));
        eq_depth_layer4=d(i);
    else
        break
    end
end
delta_eq_layer4 = eq_depth_layer4-G.Mudstone.d;
end_updated_layer4 = delta_eq_layer4+G.Gravel.d;

% Repeat the procedure: calculate the integral of the ultimate soil
% resistance of the overlying layer from ground level to the equivalent
% depth,
pu_sum_layer4 = integral(pu_fun4,0,abs(end_updated_layer4));
% and iterate to find the same integral for the underlying soil layer:
pu_sum_layer5 =0; % Initialize
for i = 1:length(d)
    if pu_sum_layer5<=pu_sum_layer4
        pu_sum_layer5 = integral(pu_fun5,0,abs(d(i)));
        eq_depth_layer5=d(i);
    else
        break
    end
end
delta_eq_layer5 = eq_depth_layer5-G.Gravel.d;
end_updated_layer5 = delta_eq_layer5+G.Sandstone.d;

% Calculate the ultimate soil resistance along the depth of the profile for
% all equivalent depths:
for i = 1:length(d)
    % LAYER 1: SAND
    if abs(d(i,:))< abs(G.Sand.d)
        d_equivalent(i,:) = d(i,:);
        pu(i,:) = pu_fun1(abs(d_equivalent(i,:)));
    end
    % LAYER 2: SOFT CLAY
    if abs(d(i,:))>= abs(G.Sand.d) && abs(d(i,:))< abs(G.Clay.d)
        d_equivalent(i,:) = d(i,:)+delta_eq_layer2;
        pu(i,:) = pu_fun2(abs(d_equivalent(i,:)));
    end
    % LAYER 3: MUDSTONE
    if abs(d(i,:))>= abs(G.Clay.d) && abs(d(i,:))< abs(G.Mudstone.d)
        d_equivalent(i,:) = d(i,:)+delta_eq_layer3;
        pu(i,:) = pu_fun3(abs(d_equivalent(i,:)));
    end
    % LAYER 4: GRAVEL
    if abs(d(i,:))>= abs(G.Mudstone.d) && abs(d(i,:))< abs(G.Gravel.d)
        d_equivalent(i,:) = d(i,:)+delta_eq_layer4;
        pu(i,:) = pu_fun4(abs(d_equivalent(i,:)));
    end
    % LAYER 5: SANDSTONE
    if abs(d(i,:))>= abs(G.Gravel.d) && abs(d(i,:))<= abs(G.Sandstone.d)
        d_equivalent(i,:) = d(i,:)+delta_eq_layer5;
        pu(i,:) = pu_fun5(abs(d_equivalent(i,:)));
    end
end

% Only store the equivalent depths AND ultimate soil resistances corresponding to FEA elements
for w = 0:FEA.E_embed
    index = find(round(d(:,1),3)==round(-w*FEA.l_embed,3));
    store1(w+1,:) = d_equivalent(index,1);
    store2(w+1,:) = pu(index,1);
end
d_equivalent = store1; % contains values for the embedded nodes only
pu = store2; % contains values for the embedded nodes only

% Restore depth profile to actual depth profile along embedded pile
d = transpose(0:-FEA.l_embed:S.h_mp_bottom); % contains values for the embedded nodes only

% Assign a value to ALL structural nodes
% Equivalent height vector FEA.H_n_equivalent
H_n_equivalent = [FEA.H_n(1:(FEA.E_unsubm+FEA.E_subm));d_equivalent]; % FROM HUB HEIGHT TO END MP
% Ultimate soil resistance
pu = [zeros((FEA.E_unsubm+FEA.E_subm),1);pu]; % FROM HUB HEIGHT TO END MP

%% P-y curves
% The value of p is assumed positive with positive pile motion (+y to the
% right). As the model prescribes that the springs are distributed along
% the LE of the pile (right side), 

% Initialize
p = zeros(length(FEA.H_n),size(y,2)); % Soil resistance at every structural node, for every pile displacement
k_ir = zeros(length(FEA.H_n),1); % Rock initial modulus of subgrade reaction

for z = 1:size(y,1)
    if z <= (FEA.E_unsubm+FEA.E_subm+1) % not embedded and first embedded node. Assume that the soil resistance above and at the mudline surface is zero.
        p(z,:) = 0;
    else
        %% Sand
        if FEA.H_n(z)> G.Sand.d
            for j = 1:size(y,2)
                % P-y relations are defined for positive pile displacement.
                % Make sure negative pile displacement leads to negative
                % soil pressure resistance:
                if sign(y(z,j)) == -1
                    p(z,j)=-(G.Sand.A*pu(z)*tanh((G.Sand.k*abs(H_n_equivalent(z)))/(G.Sand.A*...
                        pu(z))*abs(y(z,j))));
                else
                    p(z,j)=G.Sand.A*pu(z)*tanh((G.Sand.k*abs(H_n_equivalent(z)))/(G.Sand.A*...
                        pu(z))*y(z,j));
                end
            end
        end
        %% Soft clay, cyclic, below the water table
        if FEA.H_n(z)<= G.Sand.d && FEA.H_n(z)>G.Clay.d
            for j = 1:size(y,2)
                z_r2 = 6*G.Clay.sigma_u*D/(G.Clay.gamma*D+G.Clay.J*G.Clay.sigma_u);
                y50_2 = 2.5*G.Clay.e50*D;
                k_clay = 10*pu(z)/(D*(G.Clay.e50)^(1/4));
%                 if abs(y(z,j))<=0.003
%                     if sign(y(z,j)) == -1
%                         p(z,j) = -k_clay*abs(y(z,j));
%                     else
%                         p(z,j) = k_clay*y(z,j);
%                     end
%                 else
                    if abs(H_n_equivalent(z))<z_r2
                        if abs(y(z,j))<=3*y50_2
                            if sign(y(z,j)) == -1
                                p(z,j)=-(0.5*pu(z)*(abs(y(z,j))/y50_2)^(1/3));
                            else
                                p(z,j)=0.5*pu(z)*(y(z,j)/y50_2)^(1/3);
                            end
                        elseif abs(y(z,j))>3*y50_2 && abs(y(z,j))<=15*y50_2
                            if sign(y(z,j)) == -1
                                p(z,j)= -(3^(1/3)*(((abs(H_n_equivalent(z))/z_r2)-pu(z))*abs(y(z,j))/(24*y50_2)+5/8*pu(z)-1/8*abs(H_n_equivalent(z))/z_r2));
                            else
                                p(z,j)= 3^(1/3)*(((abs(H_n_equivalent(z))/z_r2)-pu(z))*y(z,j)/(24*y50_2)+5/8*pu(z)-1/8*abs(H_n_equivalent(z))/z_r2);
                            end
                        elseif abs(y(z,j))>15*y50_2
                            if sign(y(z,j)) == -1
                                p(z,j)=-(0.5*3^(1/3)*pu(z)*abs(H_n_equivalent(z))/z_r2);
                            else
                                p(z,j)=0.5*3^(1/3)*pu(z)*abs(H_n_equivalent(z))/z_r2;
                            end
                        end
                    elseif abs(H_n_equivalent(z))>=z_r2
                        if abs(y(z,j))<=3*y50_2
                            if sign(y(z,j)) == -1
                                p(z,j)=-(0.5*pu(z)*(abs(y(z,j))/y50_2)^(1/3));
                            else
                                p(z,j)=0.5*pu(z)*(y(z,j)/y50_2)^(1/3);
                            end
                        elseif abs(y(z,j))>3*y50_2
                            if sign(y(z,j)) == -1
                                p(z,j)=-(0.5*3^(1/3)*pu(z));
                            else
                                p(z,j)=0.5*3^(1/3)*pu(z);
                            end
                        end
                    end
                %end
                if abs(p(z,j))>=abs(k_clay*y(z,j))
                    p(z,j) = k_clay*y(z,j);
                end
            end
        end
        %% Mudstone (weak rock)
        if FEA.H_n(z)<= G.Clay.d && FEA.H_n(z)> G.Mudstone.d
            if abs(H_n_equivalent(z))<=3*D
                k_ir(z) = 100+400*abs(H_n_equivalent(z))/(3*D);
            elseif abs(H_n_equivalent(z))>3*D
                k_ir(z) = 500;
            end
            Kir = k_ir*G.Mudstone.E_ir;
            yrm3 = G.Mudstone.krm*D;
            for k = 1:size(Kir,1)
                if Kir(k)==0
                    y_A(k) = 0;
                else
                    y_A(k) = (pu(k)/(2*yrm3^(1/4)*Kir(k)))^(4/3);
                end
            end
            for j = 1:size(y,2)
                k_mudstone = 40*pu(z)/(D*(G.Mudstone.e50)^(1/4));
%                 if abs(y(z,j))<=0.003
%                     if sign(y(z,j)) == -1
%                         p(z,j) = -k_mudstone*abs(y(z,j));
%                     else
%                         p(z,j) = k_mudstone*y(z,j);
%                     end
%                 else
                    if abs(y(z,j))<=y_A(z)
                        if sign(y(z,j)) == -1
                            p(z,j) = -(Kir(z)*abs(y(z,j)));
                        else
                            p(z,j) = Kir(z)*y(z,j);
                        end
                    elseif abs(y(z,j))>y_A(z)
                        if sign(y(z,j)) == -1
                            p(z,j) = -(pu(z)/2*(abs(y(z,j))/yrm3)^(0.25));
                            if abs(p(z,j))>pu(z)
                                p(z,j) = -pu(z);
                            end
                        else
                            p(z,j) = pu(z)/2*(y(z,j)/yrm3)^(0.25);
                            if p(z,j)>pu(z)
                                p(z,j) = pu(z);
                            end
                        end
                        
                    end
                %end
                if abs(p(z,j))>=abs(k_mudstone*y(z,j))
                    p(z,j) = k_mudstone*y(z,j);
                end
            end
        end
        %% Gravel (sand)
        if FEA.H_n(z)<= G.Mudstone.d && FEA.H_n(z)> G.Gravel.d
            for j = 1:size(y,2)
                if sign(y(z,j)) == -1
                    p(z,j)=-(G.Gravel.A*pu(z)*tanh((G.Gravel.k*abs(H_n_equivalent(z)))/(G.Gravel.A*...
                        pu(z))*abs(y(z,j))));
                else
                    p(z,j)=G.Gravel.A*pu(z)*tanh((G.Gravel.k*abs(H_n_equivalent(z)))/(G.Gravel.A*...
                        pu(z))*y(z,j));
                end
            end
        end
        %% Sandstone (weak rock)
        if FEA.H_n(z)<= G.Gravel.d && FEA.H_n(z)>=G.Sandstone.d
            if abs(H_n_equivalent(z))<=3*D
                k_ir(z) = 100+400*abs(H_n_equivalent(z))/(3*D);
            elseif abs(H_n_equivalent(z))>3*D
                k_ir(z) = 500;
            end
            Kir = k_ir*G.Sandstone.E_ir;
            yrm5 = G.Sandstone.krm*D;
            for k = 1:size(Kir,1)
                if Kir(k)==0
                    y_A(k) = 0;
                else
                    y_A(k) = (pu(k)/(2*yrm5^(1/4)*Kir(k)))^(4/3);
                end
            end
            for j = 1:size(y,2)
                k_sandstone = 40*pu(z)/(D*(G.Sandstone.e50)^(1/4));
%                 if abs(y(z,j))<=0.003
%                     if sign(y(z,j)) == -1
%                         p(z,j) = -k_sandstone*abs(y(z,j));
%                     else
%                         p(z,j) = k_sandstone*y(z,j);
%                     end
%                 else
                    if abs(y(z,j))<=y_A(z)
                        if sign(y(z,j)) == -1
                            p(z,j) = -(Kir(z)*abs(y(z,j)));
                        else
                            p(z,j) = Kir(z)*y(z,j);
                        end
                    elseif abs(y(z,j))>y_A(z)
                        if sign(y(z,j)) == -1
                            p(z,j) = -(pu(z)/2*(abs(y(z,j))/yrm5)^(0.25));
                            if abs(p(z,j))>pu(z)
                                p(z,j) = -pu(z);
                            end
                        else
                            p(z,j) = pu(z)/2*(y(z,j)/yrm5)^(0.25);
                            if p(z,j)>pu(z)
                                p(z,j) = pu(z);
                            end
                        end
                    end
                %end
                if abs(p(z,j))>=abs(k_sandstone*y(z,j))
                    p(z,j) = k_sandstone*y(z,j);
                end
            end
        end
    end
    
end

% %% Plots
% % Plots are only generated once when running the main code
% count = 0;
% while count < 1
% % Redefine variables for the plots
% pu_plot = pu(FEA.N_unsubm+FEA.E_subm:end);     
% % Plot the variation of ultimate soil resistance with depth
% plotpu_SSI1 = figure;
% plot_pu = plot(pu_plot/1000000,d,'k');
% plot_pu.LineWidth = 1;
% set(gca,'TickLabelInterpreter','Latex','Fontsize',11)
% xlim([0 60])
% xlabel('Ultimate soil resistance per unit length $\frac{p_u}{l}$ [MN/m]',...
%     'Interpreter','Latex','Fontsize',11)
% ylabel('Depth $d$ [m]', 'Interpreter','Latex','Fontsize',11)
% % Lower boundary of the sand layer
% d1 = G.Sand.d; 
% hline = refline([0 d1]);
% hline.Color = 'k';
% hline.LineStyle='--';
% txt1 = 'Sand';
% text(40,G.Sand.d/2,txt1,'Interpreter','Latex',...
%     'Fontsize',11);
% % Lower boundary of the clay layer
% d2 = G.Clay.d; 
% hline = refline([0 d2]);
% hline.Color = 'k';
% hline.LineStyle='--';
% txt2 = 'Soft clay';
% text(40,G.Sand.d+(G.Clay.d-G.Sand.d)/2,txt2,'Interpreter','Latex',...
%     'Fontsize',11);
% % Lower boundary of the mudstone layer
% d3 = G.Mudstone.d; 
% hline = refline([0 d3]);
% hline.Color = 'k';
% hline.LineStyle='--';
% txt3 = 'Mudstone';
% text(40,G.Clay.d+(G.Mudstone.d-G.Clay.d)/2,txt3,'Interpreter','Latex',...
%     'Fontsize',11);
% % Lower boundary of the gravel layer
% d4 = G.Gravel.d; 
% hline = refline([0 d4]);
% hline.Color = 'k';
% hline.LineStyle='--';
% txt4 = 'Gravel';
% text(40,G.Mudstone.d+(G.Gravel.d-G.Mudstone.d)/2,txt4,'Interpreter','Latex',...
%     'Fontsize',11);
% % Last soil layer
% txt5 = 'Sandstone';
% text(40,G.Gravel.d+(G.Sandstone.d-G.Gravel.d)/2,txt5,'Interpreter','Latex',...
%     'Fontsize',11);
% saveas(plotpu_SSI1,'/Users/valentinevandeputte/Documents/DATA/EWEM 3rd MSc/Thesis/Thesis/Simulations/MATLAB codes/FEA/Figures/Pu_ElasticNonlinear_Soil1','fig')
% saveas(plotpu_SSI1,'/Users/valentinevandeputte/Documents/DATA/EWEM 3rd MSc/Thesis/Thesis/Simulations/MATLAB codes/FEA/Figures/Pu_ElasticNonlinear_Soil1','epsc')
% 
% % Plot the different p-y curves for varying depth
% plotpy_SSI1_sand = figure;
% % At depth = 0m, the resistance is zero (not visible in plot). So take next
% % element:
% s1=plot(y(FEA.N_unsubm+FEA.E_subm+1,:),p(FEA.N_unsubm+FEA.E_subm+1,:)/1E6,'-.k');
% hold on
% % At end sand layer:
% s2=plot(y(find(FEA.H_n<=G.Sand.d,1)-1,:),p(find(FEA.H_n<=G.Sand.d,1)-1,:)/1E6,'-k');
% set(gca,'TickLabelInterpreter','Latex','Fontsize',11)
% xlabel('Pile deflection $y$ [m]', 'Interpreter','Latex','Fontsize',11)
% ylabel('Soil resistance per unit length $p$ [MN/m]', 'Interpreter',...
%     'Latex','Fontsize',11)
% hline = refline([0 0]);
% hline.Color = 'k';
% vline = line([0 0],[ylim]);
% vline.Color = 'k';
% legend([s1,s2],{strcat('$d$ = ',num2str(FEA.H_n(FEA.N_unsubm+FEA.E_subm+1)),'\,m'),strcat('$d$ =',num2str(FEA.H_n(find(FEA.H_n<=G.Sand.d,1)-1)),'\,m')},'Location','southeast','Interpreter','Latex','Fontsize',11);
% hold off
% saveas(plotpy_SSI1_sand,'/Users/valentinevandeputte/Documents/DATA/EWEM 3rd MSc/Thesis/Thesis/Simulations/MATLAB codes/FEA/Figures/Py_ElasticNonlinear_Soil1_Sand','fig')
% saveas(plotpy_SSI1_sand,'/Users/valentinevandeputte/Documents/DATA/EWEM 3rd MSc/Thesis/Thesis/Simulations/MATLAB codes/FEA/Figures/Py_ElasticNonlinear_Soil1_Sand','epsc')
% 
% plotpy_SSI1_clay = figure;
% % At depth start clay layer:
% c1=plot(y(find(FEA.H_n<=G.Sand.d,1),:),p(find(FEA.H_n<=G.Sand.d,1),:)/1E6,'-.k');
% hold on
% % At depth end clay layer:
% c2=plot(y(find(FEA.H_n<=G.Clay.d,1)-1,:),p(find(FEA.H_n<=G.Clay.d,1)-1,:)/1E6,'-k');
% set(gca,'TickLabelInterpreter','Latex','Fontsize',11)
% xlabel('Pile deflection $y$ [m]', 'Interpreter','Latex','Fontsize',11)
% ylabel('Soil resistance per unit length $p$ [MN/m]', 'Interpreter',...
%     'Latex','Fontsize',11)
% hline = refline([0 0]);
% hline.Color = 'k';
% vline = line([0 0],[ylim]);
% vline.Color = 'k';
% legend([c1,c2],{strcat('$d$ = ',num2str(FEA.H_n(find(FEA.H_n<=G.Sand.d,1))),'\,m'),strcat('$d$ =',num2str(FEA.H_n(find(FEA.H_n<=G.Clay.d,1)-1)),'\,m')},'Location','southeast','Interpreter','Latex','Fontsize',11);
% hold off
% saveas(plotpy_SSI1_clay,'/Users/valentinevandeputte/Documents/DATA/EWEM 3rd MSc/Thesis/Thesis/Simulations/MATLAB codes/FEA/Figures/Py_ElasticNonlinear_Soil1_Clay','fig')
% saveas(plotpy_SSI1_clay,'/Users/valentinevandeputte/Documents/DATA/EWEM 3rd MSc/Thesis/Thesis/Simulations/MATLAB codes/FEA/Figures/Py_ElasticNonlinear_Soil1_Clay','epsc')
% 
% plotpy_SSI1_mudstone = figure;
% % At depth start mudstone layer:
% m1=plot(y(find(FEA.H_n<=G.Clay.d,1),:),p(find(FEA.H_n<=G.Clay.d,1),:)/1E6,'-.k');
% hold on
% % At depth end mudstone layer:
% m2=plot(y(find(FEA.H_n<=G.Mudstone.d,1)-1,:),p(find(FEA.H_n<=G.Mudstone.d,1)-1,:)/1E6,'-k');
% set(gca,'TickLabelInterpreter','Latex','Fontsize',11)
% xlabel('Pile deflection $y$ [m]', 'Interpreter','Latex','Fontsize',11)
% ylabel('Soil resistance per unit length $p$ [MN/m]', 'Interpreter',...
%     'Latex','Fontsize',11)
% hline = refline([0 0]);
% hline.Color = 'k';
% vline = line([0 0],[ylim]);
% vline.Color = 'k';
% legend([m1,m2],{strcat('$d$ = ',num2str(FEA.H_n(find(FEA.H_n<=G.Clay.d,1))),'\,m'),strcat('$d$ =',num2str(FEA.H_n(find(FEA.H_n<=G.Mudstone.d,1)-1)),'\,m')},'Location','southeast','Interpreter','Latex','Fontsize',11);
% hold off
% saveas(plotpy_SSI1_mudstone,'/Users/valentinevandeputte/Documents/DATA/EWEM 3rd MSc/Thesis/Thesis/Simulations/MATLAB codes/FEA/Figures/Py_ElasticNonlinear_Soil1_Mudstone','fig')
% saveas(plotpy_SSI1_mudstone,'/Users/valentinevandeputte/Documents/DATA/EWEM 3rd MSc/Thesis/Thesis/Simulations/MATLAB codes/FEA/Figures/Py_ElasticNonlinear_Soil1_Mudstone','epsc')
% 
% plotpy_SSI1_gravel = figure;
% % At depth start gravel layer:
% g1=plot(y(find(FEA.H_n<=G.Mudstone.d,1),:),p(find(FEA.H_n<=G.Mudstone.d,1),:)/1E6,'-.k');
% hold on
% % At depth end gravel layer:
% g2=plot(y(find(FEA.H_n<=G.Gravel.d,1)-1,:),p(find(FEA.H_n<=G.Gravel.d,1)-1,:)/1E6,'-k');
% set(gca,'TickLabelInterpreter','Latex','Fontsize',11)
% xlabel('Pile deflection $y$ [m]', 'Interpreter','Latex','Fontsize',11)
% ylabel('Soil resistance per unit length $p$ [MN/m]', 'Interpreter',...
%     'Latex','Fontsize',11)
% hline = refline([0 0]);
% hline.Color = 'k';
% vline = line([0 0],[ylim]);
% vline.Color = 'k';
% legend([g1,g2],{strcat('$d$ = ',num2str(FEA.H_n(find(FEA.H_n<=G.Mudstone.d,1))),'\,m'),strcat('$d$ =',num2str(FEA.H_n(find(FEA.H_n<=G.Gravel.d,1)-1)),'\,m')},'Location','southeast','Interpreter','Latex','Fontsize',11);
% hold off
% saveas(plotpy_SSI1_gravel,'/Users/valentinevandeputte/Documents/DATA/EWEM 3rd MSc/Thesis/Thesis/Simulations/MATLAB codes/FEA/Figures/Py_ElasticNonlinear_Soil1_Gravel','fig')
% saveas(plotpy_SSI1_gravel,'/Users/valentinevandeputte/Documents/DATA/EWEM 3rd MSc/Thesis/Thesis/Simulations/MATLAB codes/FEA/Figures/Py_ElasticNonlinear_Soil1_Gravel','epsc')
% 
% plotpy_SSI1_sandstone = figure;
% % At depth start sandstone layer:
% ss1=plot(y(find(FEA.H_n<=G.Gravel.d,1),:),p(find(FEA.H_n<=G.Gravel.d,1),:)/1E6,'-.k');
% ylim([-25 25])
% hold on
% % At depth end sandstone layer:
% ss2=plot(y(find(FEA.H_n<=G.Sandstone.d,1)-1,:),p(find(FEA.H_n<=G.Sandstone.d,1)-1,:)/1E6,'-k');
% set(gca,'TickLabelInterpreter','Latex','Fontsize',11)
% xlabel('Pile deflection $y$ [m]', 'Interpreter','Latex','Fontsize',11)
% ylabel('Soil resistance per unit length $p$ [MN/m]', 'Interpreter',...
%     'Latex','Fontsize',11)
% hline = refline([0 0]);
% hline.Color = 'k';
% vline = line([0 0],[ylim]);
% vline.Color = 'k';
% legend([ss1,ss2],{strcat('$d$ = ',num2str(FEA.H_n(find(FEA.H_n<=G.Gravel.d,1))),'\,m'),strcat('$d$ =',num2str(FEA.H_n(find(FEA.H_n<=G.Sandstone.d,1)-1)),'\,m')},'Location','southeast','Interpreter','Latex','Fontsize',11);
% hold off
% saveas(plotpy_SSI1_sandstone,'/Users/valentinevandeputte/Documents/DATA/EWEM 3rd MSc/Thesis/Thesis/Simulations/MATLAB codes/FEA/Figures/Py_ElasticNonlinear_Soil1_Sandstone','fig')
% saveas(plotpy_SSI1_sandstone,'/Users/valentinevandeputte/Documents/DATA/EWEM 3rd MSc/Thesis/Thesis/Simulations/MATLAB codes/FEA/Figures/Py_ElasticNonlinear_Soil1_Sandstone','epsc')
% 
% count = count+1;
% end
end