%% SSI Analysis of a Monopile Offshore Wind Turbine Foundation Exposed to Earthquake Ground Motion
% Monopile foundation modeled as a Timoshenko beam suspended on a Winkler
% foundation of two different kinds
% 1) nonlinear elastic spring soil model (p-y curves, no damping/energy
% dissipation)
% 2) nonlinear parallel viscous damping model (Boulanger p-y elastoplastic 
% spring in parallel with radiation damper seismic shear waves)

%% Reference frame
% y is the horizontal axis, which is zero at the centerline/axis of
% symmetry of the structure and positive to the right. z is the vertical
% axis, with zero at mudline and positive upward. Displacement is positive
% to the right (+y), rotation is positive clockwise.

%% Variable structures
% All variables are grouped according to:

% DSS: Design Sea State - waves, wind, current
% EQ: Earthquake - data, processing
% G: Soil (ground) - data
% S: Structure - data
% SIM: Simulations - time, step, transient, shutdown
% SSI: Soil-structure interaction - p-y by API or Boulanger
% WT: Wind turbine - data, operation/shutdown, BEM
% FEA: Finite Element Analysis - FEM discretization of the structural properties
% DOF: Degrees of freedom per node
% FM: Force Moment - External forces and moments
% FN: Natural frequency
% MODAL: Modal reduction

%% Clear variables
clear all
clc
close all
tic

%% Add to path
addpath(fullfile(pwd,'Earthquake Data'));
warning('on','all');
addpath(fullfile(pwd,'Windturbine Data/Airfoils'));
warning('on','all');
addpath(fullfile(pwd,'Scripts'));
warning('on','all');
addpath(fullfile(pwd,'Functions'));
warning('on','all');


%% STRUCTURAL DATA
% ===========================================================================================================================================================
g = 9.81;                                                                   % [m/s2] Gravitational constant
% ===========================================================================================================================================================
% WIND TURBINE:
WT.turbineType = 'NREL5';                                                   % This code is only outfitted to analyse an NREL 5 MW turbine at hub height
WT.D_rotor = 126;                                                           % [m] Diameter rotor
WT.V_rated = 11.4;                                                          % [m/s] Rated wind speed turbine
WT.M_RNA = 110000+240000;                                                   % [kg] Rotor, hub plus nacelle mass (rotor-nacelle-assembly)
WT.rpm_min = 6.9;                                                           % [rpm] Rotor speed, cut-in
WT.rpm_max = 12.1;                                                          % [rpm] Rated rotor speed, constant till cut-out
WT.V_cutin = 3;                                                             % [m/s] Cut-in wind speed
WT.V_cutout = 25;                                                           % [m/s] Cut-out wind speed
% ===========================================================================================================================================================
% MONOPILE:
% Length:
S.L_mp_emb = 40.2;	                                                        % [m] Embedded length MP
S.L_mp_notemb = 29.3;                                                       % [m] Unembedded length MP
S.L_mp = S.L_mp_emb+S.L_mp_notemb;                                          % [m] Total length MP
% Height wrt reference frame:
S.h_mp_bottom = -S.L_mp_emb;                                                % [m] Depth below mudline where pile starts, z=0 at mudline
S.h_mp_taper = 7.6;                                                         % [m] Height above mudline where the taper starts
S.h_mp_top = S.L_mp_notemb;                                                 % [m] Height above mudline where pile ends
% Diameter:
S.D_mp_1 = 8;                                                               % [m] Diameter of the untapered pile
S.D_mp_2 = 5.73;                                                            % [m] Smallest diameter tapered pile
S.taper_mp = (S.D_mp_2-S.D_mp_1)/(S.h_mp_top-S.h_mp_taper);                 % [m] Variation of diameter with length
% Thickness
S.t_mp = 0.074; %[m]
% Elasticity and strength:
S.E_mp = 210*1E9;                                                           % [Pa] Young's modulus of elasticity
S.G_mp = 80*1E9;                                                            % [Pa] Shear modulus
S.v_P_mp = S.E_mp/(2*S.G_mp)-1;                                             % [-] Poisson ratio
S.yield_strength_mp = 355*10^6;                                             % [Pa] Yield strength
% Density:
S.rho_mp = 7850;                                                            % [kg/m3] Density
% ===========================================================================================================================================================
% TRANSITION PIECE:
% Height wrt reference frame:
S.h_tp_bottom = 17.8;                                                       % [m] Height wrt z-axis where TP starts
S.h_tp_taper = 30.8;                                                        % [m] Height wrt z-axis where TP taper ends
S.h_tp_top = 46.4;                                                          % [m] Height wrt z-axis where TP ends
% Length:
S.L_tp = S.h_tp_top-S.h_tp_bottom;                                          % [m] Total length TP
% Diameter:
S.D_tp_2 = 5.733087557603687;                                               % [m] Diameter of the untapered pile
S.taper_tp = S.taper_mp;                                                    % [m] Taper ratio TP = MP (smooth transition)
S.D_tp_1 = (S.h_tp_bottom-S.h_tp_taper)*S.taper_tp+S.D_tp_2;                % [m] Largest diameter tapered pile
% Thickness
S.t_tp = 0.08;                                                              % [m] Thickness TP
% Elasticity and strength
S.E_tp = 210*1E9;                                                           % [Pa] Young's modulus
S.G_tp = 80*1E9;                                                            % [Pa] Shear modulus
S.v_P_tp = S.E_tp/(2*S.G_tp)-1;                                             % [-] Poisson ratio
S.yield_strength_tp = 355*10^6;                                             % [Pa] Yield strength
% Density:
S.rho_tp = 9000;                                                            % [kg/m3] Density, increased for flange, bolts, concrete platform, inner steelwork,...
% ===========================================================================================================================================================
% TOWER:
% Height wrt to reference frame
S.h_tower_bottom = S.h_tp_top;                                              % [m] Height wrt z-axis where tower starts
S.h_tower_top = S.h_tp_top+5.6+WT.D_rotor/2;                                   % [m] Height wrt z-axis where tower/taper ends, assumed equal to hub height
% Length:
S.L_tower = S.h_tower_top-S.h_tp_top;                                       % [m] Total length tower
% Diameter:
S.D_tower_1 = S.D_tp_2;                                                     % [m] Largest diameter tapered tower, equal to TP top diameter
S.D_tower_2 = 3.87;                                                         % [m] Smallest diameter tapered tower, according to NREL 5 specifications
S.taper_tower = (S.D_tower_2-S.D_tower_1)/(S.h_tower_top-S.h_tower_bottom); % [m] Taper ratio tower
% Thickness:
S.t_tower = 0.05;                                                           % [m] Thickness tower
% Elasticity and strength:
S.E_tower = 210*1E9;                                                        % [Pa] Young's modulus
S.G_tower = 80*1E9;                                                         % [Pa] Shear modulus
S.v_P_tower = S.E_tower/(2*S.G_tower)-1;                                    % [-] Poisson ratio
S.yield_strength_tower = 355*10^6;                                          % [Pa] Yield strength tower
% Density:
S.rho_tower = 8500;                                                         % [kg/m3] Density increased for bolts, flanges,...
% ===========================================================================================================================================================
% Total structural length
S.L_total = S.L_mp_emb+S.h_tp_bottom+S.L_tp+S.L_tower;                      % [m]

%% DESIGN SEA STATE
% A normal sea state is used in the entire analysis, as this scenario is
% assumed most probable (normal sea state + earthquake).
% ===========================================================================================================================================================
% SEA:
DSS.d_hydro = 22.395;                                                       % [m] Design water depth measured from mudline
DSS.rho_hydro = 1025;                                                       % [kg/m3] Water density
DSS.u_current = 0.20;                                                       % [m/s] Mean current speed + 2*stand.dev.
DSS.Hs = 1.14;                                                              % [m] Significant wave height (NSS), from BMT Argoss data
DSS.Tp = 5.69;                                                              % [s] Peak period (NSS), from BMT Argoss data
DSS.nu_hydro = 1.83*(1E-6);                                                 % [m2/s] Kinematic viscosity seawater
DSS.CM = 2.0447;                                                            % [-] MacCamy Fuchs corrected inertia coefficient
DSS.CA = DSS.CM-1;                                                          % [-] Added mass coefficient
DSS.CD_hydro = 0.65;                                                        % [-] Drag coefficient [API, Journee2001]
% ===========================================================================================================================================================
% WIND:
DSS.V_0 = 8.32;                                                             % [m/s] Wind speed at reference height (MSL+10m) for NSS, from BMT Argoss data
DSS.rho_air = 1.244;                                                        % [kg/m3] Density air
DSS.nu_air = 15.3*(1E-6);                                                   % [m2/s] Kinematic viscosity air
DSS.z_0 = 0.0002;                                                           % [-] Offshore surface roughness
DSS.CD_air = 0.6;                                                           % [-] Drag coefficient according to CD-Re relation [Mutlu2006]

%% SOIL CONDITIONS
% ===========================================================================================================================================================
% Sand layer
G.Sand.d = -5.7;                                                            % [m] depth below mudline, end of the layer
G.Sand.gamma = 8.8*10^3;                                                    % [N/m3] Effective unit weight
G.Sand.rho = 8.8*10^3/g;                                                    % [kg/m3] Effective unit weight/g, equal to drained density
G.Sand.rho_bulk = G.Sand.rho+DSS.rho_hydro;                                 % [kg/m3] Bulk unit weight/g, equal to bulk density
G.Sand.Vs = 200;                                                            % [m/s] Shear wave velocity. Should correspond to value in DEEPSOIL for SRA
G.Sand.phi = 25;                                                            % [degree] Friction angle
G.Sand.C1 = 1.25; G.Sand.C2 = 2; G.Sand.C3 = 15;                            % [-] Constants for the construction of the p-y curve, as a function of the friction angle (see API RP 2A-WSD Dec 2000, p. 65).
G.Sand.k = 7.5*4.448222/0.0254^3;                                           % [lb/in3] to [N/m3], Initial modulus of subgrade reaction, no Vesic correction (see API RP 2A-WSD Dec 2000, p. 65).
G.Sand.A = 0.9;                                                             % [-] Cyclic p-y curve. For monotonic, change to (3-0.8*depth/D)>=0.9.
% ===========================================================================================================================================================
% Clay layer
G.Clay.d = -13.0;
G.Clay.gamma = 9.4*10^3;                                                   
G.Clay.rho = 9.4*10^3/g;
G.Clay.rho_bulk = G.Clay.rho+DSS.rho_hydro;
G.Clay.Vs = 290;                                                           
G.Clay.sigma_u = 55*10^3;                                                   % [Pa] Undrained shear strength, classified as medium soft (sigma_u <= 75,000 N/m2)
G.Clay.e50 = 1.00;                                                          % [-] 50% strain         
G.Clay.J = 0.25;                                                            % According to the chart by Groundforce Shorco and the engineering properties of materials by UTexas the clay is medium, not soft: J=0.25, not 0.5. (see "laterally loaded piles rocscience.com": Peck et al. 1974)
% ===========================================================================================================================================================
% Mudstone layer
G.Mudstone.d = -33.3;
G.Mudstone.gamma = 9.9*10^3;
G.Mudstone.rho = 9.9*10^3/g;
G.Mudstone.rho_bulk = G.Mudstone.rho+DSS.rho_hydro;
G.Mudstone.Vs = 540;
G.Mudstone.sigma_u = 150*10^3;     
G.Mudstone.e50 = 0.018;           
G.Mudstone.q_ur = 2*G.Mudstone.sigma_u;                                     % [Pa] Unconfined compressive strength
G.Mudstone.E_ir = 300*G.Mudstone.q_ur;                                      % [Pa] Initial rock modulus according to Hoek & Diederichs (2006)
G.Mudstone.krm = 0.007;                                                     % [-] Constant for the overall stiffness, changed from 0.00005
% ===========================================================================================================================================================
% Gravel
G.Gravel.d = -43.0;
G.Gravel.gamma = 10*10^3;
G.Gravel.rho = 10*10^3/g;
G.Gravel.rho_bulk = G.Gravel.rho+DSS.rho_hydro;
G.Gravel.Vs = 375;
G.Gravel.phi = 32;              
G.Gravel.C1 = 2.4; G.Gravel.C2 = 3; G.Gravel.C3= 37;
G.Gravel.k = 50*4.448222/0.0254^3;
G.Gravel.A = 0.9;
% ===========================================================================================================================================================
% Sandstone
G.Sandstone.d = -50;
G.Sandstone.gamma = 9.9*10^3;
G.Sandstone.rho = 9.9*10^3/g;
G.Sandstone.rho_bulk = G.Sandstone.rho+DSS.rho_hydro;
G.Sandstone.Vs = 700;
G.Sandstone.sigma_u = 265*10^3;     
G.Sandstone.e50 = 0.018;           
G.Sandstone.q_ur = 2*G.Sandstone.sigma_u;      
G.Sandstone.E_ir = 350*G.Sandstone.q_ur;       
G.Sandstone.krm = 0.007;
% ===========================================================================================================================================================
% Rock strength reduction factors (mudstone & sandstone)
G.Mudstone.alpha_r= 1-2/3*(abs(G.Mudstone.d)-abs(G.Clay.d))/...             % [-] Strength reduction factor for weak rock
    abs(G.Sandstone.d); 
G.Sandstone.alpha_r= 1-2/3*(abs(G.Sandstone.d)-abs(G.Gravel.d))/...
    abs(G.Sandstone.d); 

%% EARTHQUAKE OCCURRENCE
% ===========================================================================================================================================================
EQ.EQ_Record = 'None';                                                      % Select 'None', 'Recorded', 'ELE' or 'ALE'
Earthquake_Signal                                                           % Load SRA DEEPSOIL analysis results. This file contains the acceleration,
                                                                            % velocity and displacement for every node along the embedded pile, over
                                                                            % time. NOTE: in case number of elements embedded pile is changed, the SRA
                                                                            % analysis should be re-executed in DEEPSOIL.



%% OPERATION ROTOR
% ===========================================================================================================================================================
WT.Operation = 'Normal'; % 'Normal' or 'Shutdown'
 
% Simulation time:
Simulation_Time

% Create regular waves (used in FEA discretization):
Regular_Waves

%% SOIL-STRUCTURE INTERACTION MODEL
% ===========================================================================================================================================================
SSI = 'SSI1';                                                               % SSI1 (API p-y curves) or SSI2 (Boulanger p-y curves + radiation damping)

%% FEA DISCRETIZATION
% ===========================================================================================================================================================
% Elements per structural section:
FEA.E_embed = 25;                                                           % Number of elements for the structure below mudline
FEA.E_subm = 20;                                                            % Number of elements for the structure below water
FEA.E_unsubm = 30;                                                          % Number of elements above water
% ! NOTE: in case number of elements embedded pile is changed, the SRA
% analysis should be re-executed in DEEPSOIL. It is advised NOT to change
% FEA.E_embed !

% Element discretization:
FEA_Discretization

%% TIMOSHENKO BEAM APPROXIMATION
% ===========================================================================================================================================================
% Degrees of freedom per node:
DOF = 2;
% Number of mode shapes considered in modal reduction dynamic analysis:
MODAL.NumberOfModeShapes = 18;
% Timoshenko FEM:
Timoshenko_Beam

%% BEAM ON A NONLINEAR WINKLER SPRING FOUNDATION
% ===========================================================================================================================================================
% Create lookup table soil stiffness
Initialize_Soil

%% MATRIX DEFINITIONS
% ===========================================================================================================================================================
FEM_Matrices

%% LOAD CASE SIMULATION
% ===========================================================================================================================================================
% Initial conditions:
IC = zeros(2*MODAL.NumberOfModeShapes,1);                                   % with modal reduction, less IC are needed! Beware that this vector contains y, ydot, theta and thetadot!!

% ODE solver:
[MODAL.t_out,MODAL.q_out] = ode45(@(t,q) EOM(DOF,MODAL.PHI,MODAL.NumberOfModeShapes,FEA,DSS,S,EQ,G,WT,WT.Operation,SSI,SIM.t_shutdown,t,q),EQ.t,IC);

%% SOLUTION PROCESSING
% Displacement and rotation history of the pile
% Modal to full
SOL.y_out = MODAL.PHI*MODAL.q_out(:,1:2:end)';                              % matrix containing y_pile and theta_pile
% Displacement pile
SOL.y_pile = SOL.y_out(1:2:end,:);                                          % matrix containing only y_pile (lateral pile motion)
% Rotation pile
SOL.theta_pile = SOL.y_out(2:2:end,:);                                      % matrix containing only theta_pile (pile rotation)

% Velocity and acceleration history of the pile displacement and rotation
for j = 1:length(EQ.t)
[SOL.DSS.eta(j),SOL.DSS.u_current(:,j),SOL.DSS.u_wave(:,j),SOL.DSS.dudt_wave(:,j),SOL.FM.f_hydro(:,j),SOL.DSS.V_air(:,j),SOL.FM.f_aero(:,j),SOL.EQ.q_eq(:,j),SOL.EQ.dqdt_eq(:,j),SOL.EQ.ddqdt_eq(:,j),SOL.FEA.M_global(:,:,j),SOL.G.kf(:,j),SOL.FEA.K_global(:,:,j),SOL.G.cv(:,j),SOL.FEA.C_global(:,:,j),SOL.FM.FMt(:,j),SOL.dydt_pile(:,j),SOL.ddydt_pile(:,j),SOL.dthetadt_pile(:,j),SOL.ddthetadt_pile(:,j)] = ODEfun(DOF,MODAL.PHI,MODAL.NumberOfModeShapes,FEA,DSS,S,EQ,G,WT,WT.Operation,SSI,SIM.t_shutdown,MODAL.t_out(j),MODAL.q_out(j,:)');
end